@@ -13,8 +13,8 @@
 ## |
 ## |  Contact us for information about our services: info@rpact.com
 ## |
-## |  File version: $Revision: 7742 $
-## |  Last changed: $Date: 2024-03-22 13:46:29 +0100 (Fr, 22 Mrz 2024) $
+## |  File version: $Revision: 8625 $
+## |  Last changed: $Date: 2025-03-21 14:46:59 +0100 (Fr, 21 Mrz 2025) $
 ## |  Last changed by: $Author: pahlke $
 ## |
 
@@ -108,7 +108,7 @@ NULL
         x <- t(x)
     }
 
-    for (i in 1:length(x)) {
+    for (i in seq_len(length(x))) {
         if (nchar(expectedResult) > 0) {
             expectedResult <- paste0(expectedResult, ", ")
         }
@@ -189,6 +189,10 @@ NULL
         return("getSimulationSurvival")
     }
 
+    if (inherits(obj, "SimulationResultsCountData")) {
+        return("getSimulationCounts")
+    }
+
     if (inherits(obj, "SimulationResultsMultiArmMeans")) {
         return("getSimulationMultiArmMeans")
     }
@@ -257,14 +261,21 @@ NULL
 
 #' @rdname getObjectRCode
 #' @export
-rcmd <- function(obj, ...,
+rcmd <- function(
+        obj, 
+        ...,
         leadingArguments = NULL,
         includeDefaultParameters = FALSE,
         stringWrapParagraphWidth = 90,
         prefix = "",
         postfix = "",
         stringWrapPrefix = "",
-        newArgumentValues = list()) {
+        newArgumentValues = list(),
+        
+        tolerance = 1e-07,
+        pipeOperator = c("auto", "none", "magrittr", "R"),
+        output = c("vector", "cat", "test", "markdown", "internal"),
+        explicitPrint = FALSE) {
     getObjectRCode(
         obj = obj,
         leadingArguments = leadingArguments,
@@ -273,7 +284,11 @@ rcmd <- function(obj, ...,
         prefix = prefix,
         postfix = postfix,
         stringWrapPrefix = stringWrapPrefix,
-        newArgumentValues = newArgumentValues
+        newArgumentValues = newArgumentValues,
+        tolerance = tolerance,
+        pipeOperator = pipeOperator,
+        output = output,
+        explicitPrint = explicitPrint
     )
 }
 
@@ -311,7 +326,9 @@ rcmd <- function(obj, ...,
 #'
 #' @export
 #'
-getObjectRCode <- function(obj, ...,
+getObjectRCode <- function(
+        obj, 
+        ...,
         leadingArguments = NULL,
         includeDefaultParameters = FALSE,
         stringWrapParagraphWidth = 90,
@@ -367,10 +384,7 @@ getObjectRCode <- function(obj, ...,
     }
 
     if (!is.null(obj) && is.function(obj)) {
-        lines <- .getFunctionAsString(obj,
-            stringWrapPrefix = stringWrapPrefix,
-            stringWrapParagraphWidth = stringWrapParagraphWidth
-        )
+        lines <- .getFunctionAsString(obj, stringWrapPrefix = stringWrapPrefix)
         if (length(lines) == 0) {
             return("")
         }
@@ -581,7 +595,7 @@ getObjectRCode <- function(obj, ...,
 
     thetaH0 <- NA_real_
     if (inherits(obj, "SimulationResultsSurvival") &&
-            obj$.getParameterType("thetaH1") == "g") {
+            obj$.getParameterType("thetaH1") == C_PARAM_GENERATED) {
         objNames <- c(objNames, "thetaH1")
         thetaH0 <- obj[["thetaH0"]]
     }
@@ -616,9 +630,16 @@ getObjectRCode <- function(obj, ...,
             }
         }
     }
+    
+    if (!("accrualTime" %in% objNames) && !is.null(obj[[".accrualTime"]]) &&
+            obj$.getParameterType("accrualTime") == C_PARAM_GENERATED &&
+            obj$.accrualTime$.getParameterType("accrualTimeOriginal") == C_PARAM_USER_DEFINED) {
+        objNames <- c(objNames, "accrualTime")
+    }
 
     if (!("accrualIntensity" %in% objNames) && !is.null(obj[[".accrualTime"]]) &&
-            !obj$.accrualTime$absoluteAccrualIntensityEnabled) {
+            !obj$.accrualTime$absoluteAccrualIntensityEnabled && 
+            obj$.getParameterType("accrualIntensity") == C_PARAM_USER_DEFINED) {
         objNames <- c(objNames, "accrualIntensity")
         objNames <- objNames[objNames != "accrualIntensityRelative"]
     }
@@ -662,76 +683,84 @@ getObjectRCode <- function(obj, ...,
         argumentsRCode <- ""
         arguments <- c()
         if (length(objNames) > 0) {
-            for (name in objNames) {
-                if (grepl("^\\.conditionalPowerResults\\$", name)) {
-                    name <- sub("^\\.conditionalPowerResults\\$", "", name)
-                    value <- obj$.conditionalPowerResults[[name]]
+            for (objName in objNames) {
+                if (grepl("^\\.conditionalPowerResults\\$", objName)) {
+                    objName <- sub("^\\.conditionalPowerResults\\$", "", objName)
+                    value <- obj$.conditionalPowerResults[[objName]]
                 } else {
-                    value <- obj[[name]]
+                    value <- obj[[objName]]
                 }
 
-                if (name == "accrualTime" && inherits(obj, "AccrualTime") &&
+                if (objName == "accrualTime" && inherits(obj, "AccrualTime") &&
                         !isTRUE(obj$endOfAccrualIsUserDefined) &&
                         isTRUE(length(obj$accrualIntensity) < length(value))) {
                     value <- value[1:(length(value) - 1)]
                 }
-
-                if (name == "accrualIntensityRelative") {
-                    name <- "accrualIntensity"
+                if (objName == "accrualTime" && !is.null(obj[[".accrualTime"]]) &&
+                        obj$.getParameterType("accrualTime") == C_PARAM_GENERATED &&
+                        obj$.accrualTime$.getParameterType("accrualTimeOriginal") == C_PARAM_USER_DEFINED) {
+                    value <- obj$.accrualTime$accrualTimeOriginal
                 }
-                if (name == "accrualIntensity" && !is.null(obj[[".accrualTime"]]) &&
+                if (objName == "accrualTimeOriginal") {
+                    objName <- "accrualTime"
+                }
+                
+                if (objName == "accrualIntensityRelative") {
+                    objName <- "accrualIntensity"
+                }
+                if (objName == "accrualIntensity" && !is.null(obj[[".accrualTime"]]) &&
                         !obj$.accrualTime$absoluteAccrualIntensityEnabled) {
                     value <- obj$.accrualTime$accrualIntensityRelative
                 }
 
                 originalValue <- value
-                newValue <- newArgumentValues[[name]]
+                newValue <- newArgumentValues[[objName]]
                 if (!is.null(newValue)) {
                     originalValue <- newValue
                 }
 
-                value <- .getArgumentValueRCode(originalValue, name)
+                value <- .getArgumentValueRCode(originalValue, objName)
 
-                if (name == "allocationRatioPlanned") {
+                if (objName == "allocationRatioPlanned") {
                     optimumAllocationRatio <- obj[["optimumAllocationRatio"]]
                     if (!is.null(optimumAllocationRatio) && isTRUE(optimumAllocationRatio)) {
                         value <- 0
                     } else if (inherits(obj, "ParameterSet")) {
-                        if (obj$.getParameterType("allocationRatioPlanned") == "g") {
+                        if (obj$.getParameterType("allocationRatioPlanned") == C_PARAM_GENERATED) {
                             value <- 0
                         }
                     }
-                } else if (name == "optimumAllocationRatio") {
-                    name <- "allocationRatioPlanned"
+                } else if (objName == "optimumAllocationRatio") {
+                    objName <- "allocationRatioPlanned"
                     value <- 0
-                } else if (name == "maxNumberOfSubjects") {
-                    value <- .getArgumentValueRCode(originalValue[1], name)
-                } else if (name == "thetaH1" && length(thetaH0) == 1 && !is.na(thetaH0) && value != 1) {
-                    value <- .getArgumentValueRCode(originalValue * thetaH0, name)
-                } else if (name == "nPlanned") {
+                } else if (objName == "maxNumberOfSubjects") {
+                    value <- .getArgumentValueRCode(originalValue[1], objName)
+                } else if (objName == "thetaH1" && length(thetaH0) == 1 && !is.na(thetaH0) && value != 1) {
+                    value <- .getArgumentValueRCode(originalValue * thetaH0, objName)
+                } else if (objName == "nPlanned") {
                     if (!all(is.na(originalValue))) {
-                        value <- .getArgumentValueRCode(na.omit(originalValue), name)
+                        value <- .getArgumentValueRCode(na.omit(originalValue), objName)
                     }
                 }
 
-                if (name == "calcSubjectsFunction" &&
+                if (objName == "calcSubjectsFunction" &&
                         obj$.getParameterType("calcSubjectsFunction") == C_PARAM_USER_DEFINED &&
                         !is.null(obj[["calcSubjectsFunction"]])) {
                     value <- "calcSubjectsFunction"
-                } else if (name == "calcEventsFunction" &&
+                } else if (objName == "calcEventsFunction" &&
                         obj$.getParameterType("calcEventsFunction") == C_PARAM_USER_DEFINED &&
                         !is.null(obj[["calcEventsFunction"]])) {
                     value <- "calcEventsFunction"
                 }
 
-                if ((name == "twoSidedPower" && isFALSE(originalValue)) || name == "accrualIntensityRelative") {
+                if ((objName == "twoSidedPower" && isFALSE(originalValue)) || objName == "accrualIntensityRelative") {
                     # do not add
                     # arguments <- c(arguments, paste0(name, "_DoNotAdd"))
                 } else {
                     if (length(value) > 0 && nchar(as.character(value)) > 0) {
-                        argument <- paste0(name, " = ", value)
+                        argument <- paste0(objName, " = ", value)
                     } else {
-                        argument <- name
+                        argument <- objName
                     }
                     if (!(argument %in% leadingArguments)) {
                         arguments <- c(arguments, argument)
@@ -740,45 +769,51 @@ getObjectRCode <- function(obj, ...,
             }
         }
 
-        if (inherits(obj, "TrialDesignPlanSurvival")) {
+        if (inherits(obj, "TrialDesignPlanSurvival") || inherits(obj, "SimulationResultsSurvival")) {
             if (!("accrualTime" %in% objNames) &&
-                    obj$.getParameterType("accrualTime") == "g" && !all(is.na(obj$accrualTime))) {
+                    obj$.getParameterType("accrualTime") == C_PARAM_GENERATED && !all(is.na(obj$accrualTime))) {
                 # case 2: follow-up time and absolute intensity given
                 accrualType2 <- (length(obj$accrualIntensity) == 1 && obj$accrualIntensity >= 1 &&
-                    obj$.getParameterType("accrualIntensity") == "u" &&
-                    obj$.getParameterType("followUpTime") == "u" &&
-                    obj$.getParameterType("maxNumberOfSubjects") == "g")
+                    obj$.getParameterType("accrualIntensity") == C_PARAM_USER_DEFINED &&
+                    obj$.getParameterType("followUpTime") == C_PARAM_USER_DEFINED &&
+                    obj$.getParameterType("maxNumberOfSubjects") == C_PARAM_GENERATED)
 
                 if (!accrualType2) {
                     accrualTime <- .getArgumentValueRCode(obj$accrualTime, "accrualTime")
                     if (length(obj$accrualTime) > 1 && length(obj$accrualTime) == length(obj$accrualIntensity) &&
-                            (obj$.getParameterType("maxNumberOfSubjects") == "u" ||
-                                obj$.getParameterType("followUpTime") == "u")) {
+                        (obj$.getParameterType("maxNumberOfSubjects") == C_PARAM_USER_DEFINED ||
+                            obj$.getParameterType("followUpTime") == C_PARAM_USER_DEFINED)) {
                         accrualTime <- .getArgumentValueRCode(obj$accrualTime[1:(length(obj$accrualTime) - 1)], "accrualTime")
                     }
-                    accrualTimeArg <- paste0("accrualTime = ", accrualTime)
-
-                    index <- which(grepl("^accrualIntensity", arguments))
-                    if (length(index) == 1 && index > 1) {
-                        arguments <- c(arguments[1:(index - 1)], accrualTimeArg, arguments[index:length(arguments)])
-                    } else {
-                        arguments <- c(arguments, accrualTimeArg)
+                    
+                    if (!identical(accrualTime, c(0, 12)) && !identical(accrualTime, c(0L, 12L)) &&
+                            getParameterType(obj$.accrualTime, "accrualTime") != C_PARAM_GENERATED) {
+                        accrualTimeArg <- paste0("accrualTime = ", accrualTime)
+                        
+                        index <- which(grepl("^accrualIntensity", arguments))
+                        if (length(index) == 1 && index > 1) {
+                            arguments <- c(arguments[1:(index - 1)], accrualTimeArg, arguments[index:length(arguments)])
+                        } else {
+                            arguments <- c(arguments, accrualTimeArg)
+                        }
                     }
-                } else if (obj$.getParameterType("followUpTime") == "u") {
+                } else if (obj$.getParameterType("followUpTime") == C_PARAM_USER_DEFINED) {
                     arguments <- c(arguments, "accrualTime = 0")
                 }
             }
 
             accrualIntensityRelative <- obj$.accrualTime$accrualIntensityRelative
-            if (!("accrualIntensity" %in% objNames) && !all(is.na(accrualIntensityRelative))) {
+            if (!("accrualIntensity" %in% objNames) && !all(is.na(accrualIntensityRelative)) && 
+                    obj$.accrualTime$.getParameterType("accrualIntensityRelative") == C_PARAM_USER_DEFINED) {
                 arguments <- c(arguments, paste0(
                     "accrualIntensity = ",
                     .getArgumentValueRCode(accrualIntensityRelative, "accrualIntensity")
                 ))
             }
 
-            if (!("maxNumberOfSubjects" %in% objNames) && obj$.accrualTime$.getParameterType("maxNumberOfSubjects") == "u" &&
-                    !(obj$.getParameterType("followUpTime") %in% c("u", "d"))) {
+            if (!("maxNumberOfSubjects" %in% objNames) && 
+                obj$.accrualTime$.getParameterType("maxNumberOfSubjects") == C_PARAM_USER_DEFINED &&
+                    !(obj$.getParameterType("followUpTime") %in% c(C_PARAM_USER_DEFINED, C_PARAM_DEFAULT_VALUE))) {
                 arguments <- c(arguments, paste0(
                     "maxNumberOfSubjects = ",
                     .getArgumentValueRCode(obj$maxNumberOfSubjects[1], "maxNumberOfSubjects")
@@ -841,21 +876,36 @@ getObjectRCode <- function(obj, ...,
             is.character(stringWrapPrefix)) {
         rCodeNew <- character()
         for (rCodeLine in rCode) {
-            rCodeLine <- gsub("   ", "___", rCodeLine)
-            rCodeLine <- gsub("  ", "__", rCodeLine)
-            rCodeLines <- strwrap(rCodeLine, width = stringWrapParagraphWidth)
+            for (i in 12:2) {
+                rCodeLine <- gsub(
+                    pattern = paste(rep(" ", i), collapse = ""), 
+                    replacement = paste(rep("_", i), collapse = ""), 
+                    x = rCodeLine)
+            }
+            if (!grepl("^__", rCodeLine)) {
+                rCodeLine <- gsub(" = ", "_=_", rCodeLine)
+                rCodeLines <- strwrap(rCodeLine, width = stringWrapParagraphWidth - 2)
+                rCodeLines <- gsub("_=_", " = ", rCodeLines)
+            } else {
+                rCodeLines <- rCodeLine
+            }
             if (length(rCodeLines) > 1) {
                 for (i in 2:length(rCodeLines)) {
                     if (grepl("^ *(\\|>|%>%) *", rCodeLines[i])) {
                         rCodeLines[i - 1] <- paste0(rCodeLines[i - 1], pipeOperatorPostfix)
                         rCodeLines[i] <- sub("^ *(\\|>|%>%) *", "", rCodeLines[i])
-                    } else if (!grepl("^ *([a-zA-Z0-9]+ *<-)|(^ *get[a-zA-Z]+\\()|summary\\(", rCodeLines[i])) {
+                    } else if (!grepl("^ *([a-zA-Z0-9]+ *<-)|(^ *get[a-zA-Z]+\\()|summary\\(", 
+                            rCodeLines[i])) {
                         rCodeLines[i] <- paste0(stringWrapPrefix, rCodeLines[i])
                     }
                 }
             }
-            rCodeLines <- gsub("___", "   ", rCodeLines)
-            rCodeLines <- gsub("__", "  ", rCodeLines)
+            for (i in 12:2) {
+                rCodeLines <- gsub(
+                    pattern = paste(rep("_", i), collapse = ""), 
+                    replacement = paste(rep(" ", i), collapse = ""), 
+                    x = rCodeLines)
+            }
             rCodeLines <- rCodeLines[nchar(trimws(rCodeLines)) > 0]
             rCodeNew <- c(rCodeNew, rCodeLines)
         }
