@@ -13,8 +13,8 @@
 ## |
 ## |  Contact us for information about our services: info@rpact.com
 ## |
-## |  File version: $Revision: 7742 $
-## |  Last changed: $Date: 2024-03-22 13:46:29 +0100 (Fr, 22 Mrz 2024) $
+## |  File version: $Revision: 8490 $
+## |  Last changed: $Date: 2025-01-20 09:58:53 +0100 (Mo, 20 Jan 2025) $
 ## |  Last changed by: $Author: pahlke $
 ## |
 
@@ -35,15 +35,15 @@
         return(plannedSubjects[stage] - plannedSubjects[stage - 1])
     }
 
-    thetaStandardized <- thetaH1 / stDevH1
-
-    mult <- 1
-    if (groups == 2) {
+    if (groups == 1) {
+        thetaStandardized <- thetaH1 / stDevH1
+    } else {
         thetaH0 <- ifelse(meanRatio, thetaH0, 1)
-        mult <- 1 + 1 / allocationRatioPlanned[stage] + thetaH0^2 * (1 + allocationRatioPlanned[stage])
+        r <- allocationRatioPlanned[stage]
+        thetaStandardized <- thetaH1 / sqrt(stDevH1[1]^2 * (1 + r) / r + thetaH0^2 * stDevH1[2]^2 * (1 + r))
     }
 
-    stageSubjects <- (max(0, conditionalCriticalValue + .getQNorm(conditionalPower)))^2 * mult /
+    stageSubjects <- (max(0, conditionalCriticalValue + .getQNorm(conditionalPower)))^2 /
         (max(1e-12, thetaStandardized))^2
 
     stageSubjects <- min(
@@ -165,7 +165,7 @@ getSimulationMeans <- function(design = NULL, ...,
         alternative = seq(0, 1, 0.2), # C_ALTERNATIVE_POWER_SIMULATION_DEFAULT
         stDev = 1, # C_STDEV_DEFAULT
         plannedSubjects = NA_real_,
-        directionUpper = TRUE, # C_DIRECTION_UPPER_DEFAULT
+        directionUpper = NA, # C_DIRECTION_UPPER_DEFAULT
         allocationRatioPlanned = NA_real_,
         minNumberOfSubjectsPerStage = NA_real_,
         maxNumberOfSubjectsPerStage = NA_real_,
@@ -186,11 +186,15 @@ getSimulationMeans <- function(design = NULL, ...,
             ), ...
         )
     } else {
-        .assertIsTrialDesign(design)
+        .assertIsTrialDesignInverseNormalOrGroupSequentialOrFisher(design)
         .warnInCaseOfUnknownArguments(functionName = "getSimulationMeans", ignore = c("showStatistics"), ...)
         .warnInCaseOfTwoSidedPowerArgument(...)
+        design <- .resetPipeOperatorQueue(design)
     }
-    .assertIsSingleLogical(directionUpper, "directionUpper")
+    directionUpper <- .assertIsValidDirectionUpper(directionUpper,
+        design,
+        objectType = "power", userFunctionCallEnabled = TRUE
+    )
     .assertIsSingleNumber(thetaH0, "thetaH0")
     if (meanRatio) {
         .assertIsInOpenInterval(thetaH0, "thetaH0", 0, NULL, naAllowed = TRUE)
@@ -207,13 +211,12 @@ getSimulationMeans <- function(design = NULL, ...,
     .assertIsSingleNumber(conditionalPower, "conditionalPower", naAllowed = TRUE)
     .assertIsInOpenInterval(conditionalPower, "conditionalPower", 0, 1, naAllowed = TRUE)
     .assertIsSingleNumber(thetaH1, "thetaH1", naAllowed = TRUE)
-    .assertIsSingleNumber(stDevH1, "stDevH1", naAllowed = TRUE)
-    .assertIsInOpenInterval(stDevH1, "stDevH1", 0, NULL, naAllowed = TRUE)
+    .assertIsValidStandardDeviation(stDevH1, groups = groups, name = "stDevH1", naAllowed = TRUE)
     .assertIsNumericVector(allocationRatioPlanned, "allocationRatioPlanned", naAllowed = TRUE)
     .assertIsInOpenInterval(allocationRatioPlanned, "allocationRatioPlanned", 0, C_ALLOCATION_RATIO_MAXIMUM, naAllowed = TRUE)
     .assertIsSinglePositiveInteger(maxNumberOfIterations, "maxNumberOfIterations", validateType = FALSE)
     .assertIsSingleNumber(seed, "seed", naAllowed = TRUE)
-    .assertIsValidStandardDeviation(stDev)
+    .assertIsValidStandardDeviation(stDev, groups = groups)
     .assertIsSingleLogical(showStatistics, "showStatistics", naAllowed = FALSE)
     .assertIsSingleLogical(normalApproximation, "normalApproximation", naAllowed = FALSE)
     .assertIsValidPlannedSubjectsOrEvents(design, plannedSubjects, parameterName = "plannedSubjects")
@@ -280,8 +283,9 @@ getSimulationMeans <- function(design = NULL, ...,
         "stDevH1", stDevH1, design$kMax > 1,
         "design is fixed ('kMax' = 1)", "Assumed effect"
     )
-    if (is.na(conditionalPower) && is.null(calcSubjectsFunction) && !is.na(stDevH1)) {
-        warning("'stDevH1' will be ignored because neither 'conditionalPower' nor ",
+    if (is.na(conditionalPower) && is.null(calcSubjectsFunction) && !all(is.na(stDevH1))) {
+        warning("'stDevH1' (", .arrayToString(stDevH1), ") will be ignored ",
+            "because neither 'conditionalPower' nor ",
             "'calcSubjectsFunction' is defined",
             call. = FALSE
         )
@@ -432,6 +436,8 @@ getSimulationMeans <- function(design = NULL, ...,
         designNumber <- 2L
     } else if (.isTrialDesignFisher(design)) {
         designNumber <- 3L
+    } else {
+        stop(C_EXCEPTION_TYPE_RUNTIME_ISSUE, "design type ", sQuote(.getClassName(design)), " not supported")
     }
 
     if (.isTrialDesignFisher(design)) {
@@ -441,7 +447,16 @@ getSimulationMeans <- function(design = NULL, ...,
         alpha0Vec <- rep(NA_real_, design$kMax - 1)
         futilityBounds <- design$futilityBounds
     }
-    if (is.na(stDevH1)) {
+
+    if ((length(stDev) == 1) && (groups == 2)) {
+        stDev <- rep(stDev, 2)
+    }
+
+    if ((length(stDevH1) == 1) && (groups == 2)) {
+        stDevH1 <- rep(stDevH1, 2)
+    }
+
+    if (all(is.na(stDevH1))) {
         stDevH1 <- stDev
     }
 
@@ -463,7 +478,7 @@ getSimulationMeans <- function(design = NULL, ...,
         informationRates = design$informationRates,
         futilityBounds = futilityBounds,
         alpha0Vec = alpha0Vec,
-        criticalValues = design$criticalValues,
+        criticalValues = .getCriticalValues(design),
         meanRatio = meanRatio,
         thetaH0 = thetaH0,
         stDev = stDev,
@@ -491,6 +506,7 @@ getSimulationMeans <- function(design = NULL, ...,
     simulationResults$overallReject <- cppResult$overallReject
     simulationResults$futilityPerStage <- cppResult$futilityPerStage
     simulationResults$futilityStop <- cppResult$futilityStop
+
     if (design$kMax > 1) {
         if (length(alternative) == 1) {
             simulationResults$earlyStop <- sum(cppResult$futilityPerStage) + sum(cppResult$rejectPerStage[1:(design$kMax - 1)])
