@@ -14,9 +14,9 @@
  *
  * Contact us for information about our services: info@rpact.com
  *
- * File version: $Revision: 7679 $
- * Last changed: $Date: 2024-03-04 15:00:35 +0100 (Mo, 04 Mrz 2024) $
- * Last changed by: $Author: wassmer $
+ * File version: $Revision: 8620 $
+ * Last changed: $Date: 2025-03-20 09:03:31 +0100 (Do, 20 Mrz 2025) $
+ * Last changed by: $Author: pahlke $
  *
  */
 
@@ -137,7 +137,7 @@ List logRankTest(NumericVector accrualTime, NumericVector survivalTime,
 		logRank = R_NegInf;
 	}
 
-	if (!directionUpper) {
+	if (!R_IsNA(directionUpper) && !directionUpper) {
 		logRank = -logRank;
 	}
 
@@ -288,7 +288,7 @@ double getEstimatedTheta(
 /**
  * Get recalculated event sizes (only for stage > 1)
  */
-double getSimulationSuvivalStageEventsCpp(
+double getSimulationSurvivalStageEventsCpp(
 		int stage,
 		double conditionalPower,
 		double thetaH0,
@@ -314,8 +314,8 @@ double getSimulationSuvivalStageEventsCpp(
 	return requiredStageEvents;
 }
 
-Rcpp::XPtr<calcEventsFunctionSurvivalPtr> getSimulationSuvivalStageEventsXPtrCpp() {
-  return(Rcpp::XPtr<calcEventsFunctionSurvivalPtr>(new calcEventsFunctionSurvivalPtr(&getSimulationSuvivalStageEventsCpp)));
+Rcpp::XPtr<calcEventsFunctionSurvivalPtr> getSimulationSurvivalStageEventsXPtrCpp() {
+  return(Rcpp::XPtr<calcEventsFunctionSurvivalPtr>(new calcEventsFunctionSurvivalPtr(&getSimulationSurvivalStageEventsCpp)));
 }
 
 NumericMatrix getSimulationStepResultsSurvival(
@@ -621,7 +621,7 @@ List getSimulationSurvivalCpp(
 		Nullable<Function> calcEventsFunctionR,
 		SEXP calcEventsFunctionCpp) {
 
-	Rcpp::XPtr<calcEventsFunctionSurvivalPtr> calcEventsFunctionCppXPtr = getSimulationSuvivalStageEventsXPtrCpp();
+	Rcpp::XPtr<calcEventsFunctionSurvivalPtr> calcEventsFunctionCppXPtr = getSimulationSurvivalStageEventsXPtrCpp();
 	if (calcEventsFunctionType == 0) {
 		calcEventsFunctionR = NULL;
 	}
